// 模拟比赛得分
const matches = [
  [1, 0], [2, 0], [2, 1], [3, 0], [3, 1], [3, 2], [4, 0], [4, 1], [4, 2], [5, 0], [5, 1], [5, 2], [99, 0],
  [0, 0], [1, 1], [2, 2], [3, 3], [99, 99],
  [0, 1], [0, 2], [1, 2], [0, 3], [1, 3], [2, 3], [0, 4], [1, 4], [2, 4], [0, 5], [1, 5], [2, 5], [0, 99]
]

const playWayFunc = {
  'bst-wl': computeWDL,
  'bst-wls': computeWDLS,
  'bst-score': computeScore,
  'bst-points': computePoints
}

// 遍历模拟比赛获取最大, 最小奖金
// rq 让球数
// ys 预设总分
export function GetPrice (allParts, rq, ys) {
  let minPrice
  let maxPrice

  // 遍历模拟比赛的各种可能
  matches.forEach(match => {
    const [min, max] = GetMinAndMax(allParts, match, rq, ys)
    if (min < minPrice || minPrice === undefined) {
      minPrice = min
    }

    if (max > maxPrice || maxPrice === undefined) {
      maxPrice = max
    }
  })

  return [minPrice, maxPrice]
}

function GetMinAndMax (allParts, match, rq, ys) {
  let minPrice = 0
  let maxPrice = 0

  // 所有组合
  allParts.forEach(parts => {
    // 每种组合 比如 3串4 是 3个2串1, 1个3串1
    const price = parts.map(part => {
      // 场次组合 比如 2串1
      return part.map(grp => {
        const { wayCode } = grp.detail
        const fn = playWayFunc[wayCode]
        return fn(grp, match, rq, ys)
      }).filter(Boolean).reduce((acc, x) => acc * x, 1)
    }).reduce((acc, x) => acc + x, 0)

    if (minPrice === 0 || minPrice > price) {
      minPrice = price
    }

    maxPrice += maxPrice
  })

  return [minPrice, maxPrice]
}

// 胜平负
function computeWDL (part, match) {
  const [x, y] = match
  const { ruleCode, odds } = part.detail

  switch (ruleCode) {
    case 'bst-win':
      return x > y ? odds : 0
    default:
      return x < y ? odds : 0
  }
}

// 让球胜平负
function computeWDLS (part, match, rq) {
  const [x, y] = match
  const { ruleCode, odds } = part.rules

  switch (ruleCode) {
    case 'bst-sp-win':
      return x + rq > y ? odds : 0
    default:
      return x + rq < y ? odds : 0
  }
}

// 比分
function computeScore (part, match, rq, ys) {
  const [x, y] = match
  const { ruleCode, odds } = part.detail

  switch (ruleCode) {
    case 'bst-big':
      return x + y > ys ? odds : 0
    default:
      return x + y < ys ? odds : 0
  }
}

// 进球数
function computePoints (part, match) {
  const [x, y] = match
  const { ruleCode, odds } = part.rules

  switch (ruleCode) {
    case 'bst-w15':
      return x - y >= 1 && x - y <= 5 ? odds : 0
    case 'bst-w610':
      return x - y >= 6 && x - y <= 10 ? odds : 0
    case 'bst-w1115':
      return x - y >= 11 && x - y <= 15  ? odds : 0
    case 'bst-w1620':
      return x - y >= 16 && x - y <= 20 ? odds : 0
    case 'bst-w2125':
      return x - y >= 21 && x - y <= 25 ? odds : 0
    case 'bst-w99':
      return x - y >= 26 ? odds : 0
    case 'bst-l15':
      return y - x >= 1 && y - x <= 5 ? odds : 0
    case 'bst-l610':
      return y - x >= 6 && y - x <= 10 ? odds : 0
    case 'bst-l1115':
      return y - x >= 11 && y - x <= 15  ? odds : 0
    case 'bst-l1620':
      return y - x >= 16 && y - x <= 20 ? odds : 0
    case 'bst-l2125':
      return y - x >= 21 && y - x <= 25 ? odds : 0
    case 'bst-l99':
      return y - x >= 26 ? odds : 0
    default:
      return 0
  }
}
